//
//  Pearl.m
//  Spring
//
//  Created by Mark Meretzky on 11/20/13.
//  Copyright (c) 2013 Mark Meretzky. All rights reserved.
//

#import "Pearl.h"

@implementation Pearl
@synthesize location;

- (id) initWithFrame: (CGRect) frame
{
	self = [super initWithFrame: frame];
	if (self) {
		// Initialization code
	}
	return self;
}

- (id) initWithLocation: (CGPoint) p
{
	self = [super init];
	if (self) {
		location = p;
		velocity = CGPointZero;
	}
	return self;
}

//Drag the Pearl towards point p.

- (void) drag: (CGPoint) p {
	static const CGFloat stiffness = 0.2;	//elasticity
	static const CGFloat gravity = 3;
	static const CGFloat mass = 2;	//must be positive
	static const CGFloat damping = 0.7;
	static const CGFloat radius = 20;	//in pixels

	CGPoint force = CGPointMake(
		(p.x - location.x) * stiffness,
		(p.y - location.y) * stiffness + gravity * mass
	);

	//F = ma
	CGPoint acceleration = CGPointMake(
		force.x / mass,
		force.y / mass
	);

	velocity = CGPointMake(
		damping * (velocity.x + acceleration.x),
		damping * (velocity.y + acceleration.y)
	);

	location = CGPointMake(
		location.x + velocity.x,
		location.y + velocity.y
	);
				
	CGRect rect = CGRectMake(
		location.x - radius,
		location.y - radius,
		2 * radius,
		2 * radius
	);

	CGContextRef c = UIGraphicsGetCurrentContext();

	CGContextBeginPath(c);
	CGContextAddEllipseInRect(c, rect);
	CGContextSetRGBFillColor(c, 1, 1, 1, 1);	//white, opaque
	CGContextFillPath(c);

	CGContextMoveToPoint(c, location.x, location.y);
	CGContextAddLineToPoint(c, p.x, p.y);
	CGContextSetRGBStrokeColor(c, 1, 1, 1, 1);
	CGContextStrokePath(c);
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void) drawRect: (CGRect) rect
{
	// Drawing code
}
*/

@end
